package project;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;

import exceptions.BidBelowZeroException;

public class AuctionManager {

	private Auction auction;

	public AuctionManager() {

		this.auction = new Auction();

	}

	public void setAuction(Auction a) {

		this.auction = a;

	}

	public static AuctionManager getInstance() {

		return new AuctionManager();

	}

	public boolean placeBid(User user, int value) throws BidBelowZeroException, ParseException {

		LocalDateTime l = LocalDateTime.now();

		Date d = AuctionManager.parseDate(l.format(DateTimeFormatter.ofPattern("dd/MM/yyyy HH:mm:ss")));

		if (value < 1) {

			throw new BidBelowZeroException();

		} else if (checkTime(d)) {

			Bid b = new Bid(d, value, user, auction);
			computePrice(b);

			ArrayList<Bid> toSet = auction.getBids();
			toSet.add(b);

			auction.setBids(toSet);
			auction.setCurrentLeader(user);

			return true;

		}

		return false;

	}

	public boolean checkTime(Date d) {

		// Checks for existing bids with the same time value

		ArrayList<Bid> toCheck = auction.getBids();

		for (Bid b : toCheck) {

			if (d.equals(b.getDate())) {

				return false;

			}

		}
		
		if (d.before(auction.getEnd())) {

			return true;

		}

		return false;
	}

	public void computePrice(Bid bid) {

		auction.setPrice(bid.getValue() + auction.getPrice());

	}

	public static Date parseDate(String value) {

		Date date = null;
		String format = "dd/MM/yyyy HH:mm:ss";

		SimpleDateFormat sdf = new SimpleDateFormat(format);

		try {

			date = sdf.parse(value);

			if (value.equals(sdf.format(date))) {

				return date;

			} else {

				return null;

			}

		} catch (ParseException ex) {

			return null;

		}

	}

}
