package unitTest;

import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import java.util.function.Function;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.api.function.ThrowingConsumer;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import project.Auction;
import project.Category;
import project.Item;
import project.User;

public class AuctionUnitTest {

	@BeforeEach
	public void reset() {

		Item.items = new ArrayList<Item>();
		User.users = new ArrayList<User>();
		Auction.auctions = new ArrayList<Auction>();
		Category.categories = new ArrayList<Category>();

	}

	@ParameterizedTest
	@ValueSource(ints = { 1, 3, 5, 7 })
	public void subscribeTestFalse(int userId) {

		Auction a = Auction.getInstance();
		assertFalse(a.subscribe(userId));

	}

	@TestFactory
	public Stream<DynamicTest> subscribeTestTrue() {

		Auction a = Auction.getInstance();

		Iterator<Integer> inputGenerator = new Iterator<Integer>() {

			int userId;
			Random random = new Random();

			@Override
			public boolean hasNext() {

				return random.nextInt(100) % 10 != 0;

			}

			@Override
			public Integer next() {

				userId = (int) (random.nextDouble() * 10);
				User u = new User();
				u.setId(userId);
				return userId;
			}

		};

		Function<Integer, String> displayNameGenerator = (input) -> "user ID: " + input;

		ThrowingConsumer<Integer> testExecutor = (input) -> assertTrue(a.subscribe(input));

		return DynamicTest.stream(inputGenerator, displayNameGenerator, testExecutor);

	}

	@Test
	public void assignToUserTest() {

		Auction a = Auction.getInstance();
		User u = new User();
		Item i = new Item();
		u.setId(1);
		i.setId(1);

		assertTrue(a.assignToUser(1, 1));
		assertFalse(a.assignToUser(1, 2));
		assertFalse(a.assignToUser(2, 1));

	}

}
